/*
Copyright (C) 2007 - Nastasi Christian
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _flexGUI_h_
#define _flexGUI_h_

#include <stdio.h>
#include <FL/Fl.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Double_Window.H>

#include <Fl/Fl_Image.H>

/** 
* @file flexGUI.h
* @brief Flex-CAM GUI 
* @author Nastasi Christian
* @date 2007-12-17
*/

/** 
* @brief Flex-CAM Image Showing
*/
class FlexGUI : public Fl_Double_Window
{
private:
	unsigned int width;
	unsigned int height;
	Fl_Box *m_bMain;
	Fl_RGB_Image *m_image;
public:
	/** 
	* @brief Create the GUI window object
	* 
	* @param w Image width
	* @param h Image height
	* @param *l Window name
	*/
	FlexGUI(unsigned int w, unsigned int h, const char *l=0);     
	/** 
	* @brief Show the GUI window
	*/
	void show_gui();
	/** 
	* @brief Draw the image in the GUI window
	* 
	* @param image Image data pointer
	* @param w Image width
	* @param h Image height
	* @param channel Number of channels (1=gray, 3=color)
	* 
	* @return \c true if no error occurs, \c false otherwise.
	*/
	bool image_show(unsigned char *image, unsigned int w, unsigned int h, unsigned int channel); 
};


#endif
