/*
Copyright (C) 2007 - Nastasi Christian
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _flexUtils_h_
#define _flexUtils_h_

#include <stdio.h>
#include <jpeglib.h>

typedef struct
{
	struct jpeg_source_mgr pub;   
	unsigned char* buffer;
	unsigned int   bufsize;
} myJPEG_source_mgr;

typedef myJPEG_source_mgr* myJPEG_src_ptr;

/** 
* @file flexUtils.h
* @brief Conversion functions
* @author Nastasi Christian
* @date 2007-12-17
*/


/** 
* @brief JPEG Decompression
* 
* @param srcBuffer Source JPEG image
* @param size Source JPEG image size
* @param dstBuffer Destination buffer capable to store the decompressed image
* @param width The decompressed image width
* @param height The decompressed image height
* @param bpp The decompressed image number of byte per pixel
* 
* @return 
*/
bool flex_JPEG_decompress(unsigned char* srcBuffer, int size, unsigned char* dstBuffer, int& width, int& height, int& bpp);

/** 
* @brief Convert from RGB16 to RGB24
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void flex_rgb16_to_rgb24(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);
/** 
* @brief Convert from RGB12 to RGB24
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void flex_rgb12_to_rgb24(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);
/** 
* @brief Convert from Gray4 to Gray8
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void flex_gray4_to_gray8(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);
/** 
* @brief Convert from Gray2 to Gray8
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void flex_gray2_to_gray8(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);

#endif
